package com.muyu.vehicle.core;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.muyu.common.pool.FixedThreadPool;
import com.muyu.common.pool.ScheduledThreadPool;
import com.muyu.domain.Vehicle;
import com.muyu.service.VehicleInstanceService;
import com.muyu.service.VehicleService;
import com.muyu.vehicle.VehicleInstance;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PreDestroy;
import java.util.List;

/**
 * @author DongZeLiang
 * @version 1.0
 * @description 配置
 * @date 2023/11/9
 */
@Log4j2
@Configuration
@AllArgsConstructor
public class VehicleConfiguration implements ApplicationRunner {

    private final VehicleService vehicleService;

    private final VehicleInstanceService vehicleInstanceService;

    /**
     * 初始化加载汽车数据到内存当中
     */
    public void vehiclePageInit (){
        long startTime = System.currentTimeMillis();
        int page = 0, pageSize = 10;
        log.info("初始开始，批量从数据库当中加载数据到内存当中,每次[{}]条", pageSize);
        while (true){
            Page<Vehicle> vehiclePage = vehicleService.page(new Page<>(page++, pageSize));
            List<Vehicle> vehicleList = vehiclePage.getRecords();
            vehicleList.forEach(vehicleInstanceService::init);
            log.debug("第[{}]页，[{}]条", page, vehicleList.size());
            if (vehicleList.size() < pageSize){
                break;
            }
        }
        log.info("数据加载完成，耗时：{} MS", System.currentTimeMillis() - startTime);
    }

    @Override
    public void run (ApplicationArguments args) {
        this.vehiclePageInit();
        // 提交给线程池 一分钟 执行一次
//        ThreadPool.submit(new Thread(vehicleService::syncDb), 30);
    }

    /**
     * 关闭程序前操作
     */
    @PreDestroy
    public void destroy(){


        log.info("数据库同步");
        vehicleService.syncDb();

        log.info("下线所有车辆");
        List<VehicleInstance> onlineVehicleInstanceList = LocalContainer.getOnlineVehicleInstance();
        onlineVehicleInstanceList.forEach(VehicleInstance::closeClient);

        log.info("关闭线程池");
        ScheduledThreadPool.shutdown();
        FixedThreadPool.shutDown();

    }
}
